@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2004 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2006 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: fcron.c,v 1.73 2005/03/12 12:39:21 thib Exp thib $ */
+ /* $Id: fcron.c,v 1.77 2006/02/05 20:59:08 thib Exp thib $ */
 
 #include "fcron.h"
 
@@ -33,7 +33,7 @@
 #include "socket.h"
 #endif
 
-char rcs_info[] = "$Id: fcron.c,v 1.73 2005/03/12 12:39:21 thib Exp thib $";
+char rcs_info[] = "$Id: fcron.c,v 1.77 2006/02/05 20:59:08 thib Exp thib $";
 
 void main_loop(void);
 void check_signal(void);
@@ -76,7 +76,8 @@ char *tmp_path = "";
 /* process identity */
 pid_t daemon_pid;
 mode_t saved_umask;           /* default root umask */
-char *prog_name = NULL;         
+char *prog_name = NULL;
+char *orig_tz_envvar = NULL;
 
 /* have we got a signal ? */
 char sig_conf = 0;            /* is 1 when we got a SIGHUP, 2 for a SIGUSR1 */ 
@@ -123,7 +124,7 @@ info(void)
 {
     fprintf(stderr,
 	    "fcron " VERSION_QUOTED " - periodic command scheduler\n"
-	    "Copyright 2000-2004 Thibault Godouet <fcron@free.fr>\n"
+	    "Copyright " COPYRIGHT_QUOTED " Thibault Godouet <fcron@free.fr>\n"
 	    "This program is free software distributed WITHOUT ANY WARRANTY.\n"
             "See the GNU General Public License for more details.\n"
 	);
@@ -175,7 +176,8 @@ print_schedule(void)
 	explain(" File %s", cf->cf_user);
 	for (cl = cf->cf_line_base; cl; cl = cl->cl_next) {
 	    ftime = localtime( &(cl->cl_nextexe) );
-	    explain("  cmd %s next exec %d/%d/%d wday:%d %02d:%02d",
+	    explain("  cmd %s next exec %d/%d/%d wday:%d %02d:%02d"
+		    " (system time)",
 		    cl->cl_shell, (ftime->tm_mon + 1), ftime->tm_mday,
 		    (ftime->tm_year + 1900), ftime->tm_wday,
 		    ftime->tm_hour, ftime->tm_min); 
@@ -223,6 +225,8 @@ xexit(int exit_value)
     
     free_conf();
 
+    free(orig_tz_envvar);
+
     explain("Exiting with code %d", exit_value);
     exit (exit_value);
 
@@ -503,6 +507,9 @@ main(int argc, char **argv)
      * used in die() and die_e() functions */
     daemon_pid = getpid();
 
+    /* save the value of the TZ env variable (used for option timezone) */
+    orig_tz_envvar = strdup2(getenv("TZ"));
+
     parseopt(argc, argv);
 
     /* read fcron.conf and update global parameters */
@@ -720,14 +727,23 @@ main_loop()
 #ifdef FCRONDYN
 	gettimeofday(&tv, NULL);
 	tv.tv_sec = (stime > 1) ? stime - 1 : 0;
-	tv.tv_usec = 1000000 - tv.tv_usec;
+	/* we set tv_usec to slightly more than necessary so as
+	 * we don't wake up too early, in which case we would
+	 * have to sleep again for some time */
+	tv.tv_usec = 1001000 - tv.tv_usec;
+	/* On some systems (BSD, etc), tv_usec cannot be greater than 999999 */
+	if (tv.tv_usec > 999999)
+	    tv.tv_usec = 999999;
+	/* note: read_set is set in socket.c */
 	if((retcode = select(set_max_fd+1, &read_set, NULL, NULL, &tv)) < 0 && errno != EINTR)
-	    die_e("select return %d", errno);
+	    die_e("select returned %d", errno);
 #else
 	if (stime > 1)
 	    sleep(stime - 1);
 	gettimeofday(&tv, NULL);
-	usleep( 1000000 - tv.tv_usec );
+	/* we set tv_usec to slightly more than necessary to avoid 
+	 * infinite loop */
+	usleep( 1001000 - tv.tv_usec );
 #endif /* FCRONDYN */
 #else
 	sleep(stime);
