@@ -1,8 +1,7 @@
-
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2004 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2006 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +21,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: fcrontab.c,v 1.66 2005/02/26 15:10:11 thib Exp thib $ */
+ /* $Id: fcrontab.c,v 1.68 2006/01/11 00:49:33 thib Exp thib $ */
 
 /* 
  * The goal of this program is simple : giving a user interface to fcron
@@ -47,7 +46,7 @@
 #include "temp_file.h"
 #include "read_string.h"
 
-char rcs_info[] = "$Id: fcrontab.c,v 1.66 2005/02/26 15:10:11 thib Exp thib $";
+char rcs_info[] = "$Id: fcrontab.c,v 1.68 2006/01/11 00:49:33 thib Exp thib $";
 
 void info(void);
 void usage(void);
@@ -105,7 +104,7 @@ info(void)
 {
     fprintf(stderr,
 	    "fcrontab " VERSION_QUOTED " - user interface to daemon fcron\n"
-	    "Copyright 2000-2004 Thibault Godouet <fcron@free.fr>\n"
+	    "Copyright " COPYRIGHT_QUOTED " Thibault Godouet <fcron@free.fr>\n"
 	    "This program is free software distributed WITHOUT ANY WARRANTY.\n"
             "See the GNU General Public License for more details.\n"
 	);
@@ -927,7 +926,7 @@ parseopt(int argc, char *argv[])
     if ( user == NULL ) {
 	/* get user's name using getpwuid() */
 	if ( ! (pass = getpwuid(uid)) )
-	    die("user \"%s\" is not in passwd file. Aborting.", USERNAME);
+	    die_e("user \"%s\" is not in passwd file. Aborting.", USERNAME);
 	/* we need to strdup2 the value given by getpwuid() because we free
 	 * file->cf_user in delete_file */
 	user = strdup2(pass->pw_name);
@@ -948,7 +947,7 @@ parseopt(int argc, char *argv[])
 		asgid = pass->pw_gid;
 	    }
 	    else
-		die("user \"%s\" is not in passwd file. Aborting.", user);
+		die_e("user \"%s\" is not in passwd file. Aborting.", user);
     }
 
     if ( 
@@ -978,9 +977,7 @@ main(int argc, char **argv)
     int    retcode = 0;
     const char * const * env;
 #endif
-#ifdef USE_SETE_ID
     struct passwd *pass;
-#endif
 
     memset(buf, 0, sizeof(buf));
     memset(file, 0, sizeof(file));
@@ -990,18 +987,26 @@ main(int argc, char **argv)
     
     uid = getuid();
 
+    if ( ! (pass = getpwnam(USERNAME)) )
+	die_e("user \"%s\" is not in passwd file. Aborting.", USERNAME);
+    fcrontab_uid = pass->pw_uid;
+    fcrontab_gid = pass->pw_gid;
+
     /* get current dir */
+#ifdef USE_SETE_ID
+    if (seteuid(uid) != 0) 
+	die_e("Could not change euid to %d", uid); 
+#endif
     if ( getcwd(orig_dir, sizeof(orig_dir)) == NULL )
 	die_e("getcwd");
+#ifdef USE_SETE_ID
+    if (seteuid(fcrontab_uid) != 0) 
+	die_e("Couldn't change euid to fcrontab_uid[%d]",fcrontab_uid);
+#endif
 
     /* interpret command line options */
     parseopt(argc, argv);
 
-    if ( ! (pass = getpwnam(USERNAME)) )
-	die("user \"%s\" is not in passwd file. Aborting.", USERNAME);
-    fcrontab_uid = pass->pw_uid;
-    fcrontab_gid = pass->pw_gid;
-
 #ifdef USE_SETE_ID
 
 #ifdef HAVE_LIBPAM
